@@ -8,7 +8,7 @@ import multiprocessing as mp
 from .view import View
 
 
-__all__ = ['Proxy', 'Except', 'RemoteException']
+__all__ = ['Proxy', 'Except', 'LocalExcept', 'RemoteException']
 
 
 def make_token(name):
@@ -74,7 +74,7 @@ class Proxy(View):
     _delay = 1e-5
     _listener_process_name = None
     NOCOPY = ['_local', '_remote', '_llock', '_rlock', '_listening_val']
-    def __init__(self, instance, default=UNDEFINED, eager_proxy=True, **kw):
+    def __init__(self, instance, default=UNDEFINED, eager_proxy=True, fulfill_final=True, **kw):
         super().__init__(**kw)
         self._obj = instance
 
@@ -85,6 +85,7 @@ class Proxy(View):
 
         self._default = default
         self._eager_proxy = eager_proxy
+        self._fulfill_final = fulfill_final
         self._root = self  # isn't called when extending
 
     def __str__(self):
@@ -138,12 +139,14 @@ class Proxy(View):
             if self._listening:  # if the remote process is listening, run
                 # send and wait for a result
                 self._local.send(self._keys)
-                x, exc = self._local.recv()
-                if x == SELF:
-                    x = self._root  # root remote object without any ops
-                if exc is not None:
-                    raise exc
-                return x
+                x = self._local.recv()
+                if x is not None:
+                    x, exc = x
+                    if x == SELF:
+                        x = self._root  # root remote object without any ops
+                    if exc is not None:
+                        raise exc
+                    return x
 
         # if a default value is provided, then return that, otherwise return a default.
         default = self._default if default == UNDEFINED else default
@@ -205,8 +208,15 @@ class Proxy(View):
 
     @_listening.setter
     def _listening(self, value):
-        with self._llock:
-            self._listening_val.value = int(value)
+        # do what you've got to do to get the lock
+        while not self._llock.acquire(0):
+            if self._fulfill_final:
+                self.poll()
+            else:
+                self._remote.send(None)  # cancel
+
+        self._listening_val.value = int(value)
+        self._llock.release()
         self._listener_process_name = mp.current_process().name if value else None
 
     # remote background listening interface
@@ -253,8 +263,6 @@ class Proxy(View):
                 time.sleep(self._delay)
         finally:
             self._listening = False
-            self.process_messages()
-
 
     def wait_until_listening(self):
         while not self._listening:
@@ -270,18 +278,18 @@ class Proxy(View):
 
 
 
-class Except:
+class LocalExcept:
     '''Catch exceptions in a remote process with their traceback and send them
     back to be raised properly in the main process.'''
-    def __init__(self, *types, raises=False):
+    def __init__(self, *types, raises=False, catch_once=True):
         self._local, self._remote = mp.Pipe()
-        self.types = types or (BaseException,)
+        self.types = types or (Exception,)
         self.raises = raises
-        self.groups = {}
+        self.catch_once = catch_once
+        self._groups = {}
         self._excs = {}
 
     def __str__(self):
-        self.pull()
         return '<{} raises={} types={}{}>'.format(
             self.__class__.__name__, self.raises, self.types,
             ''.join(
@@ -290,63 +298,96 @@ class Except:
                     (' - last: ({}: {!r})'.format(type(excs[-1]).__name__, str(excs[-1]))
                      if excs else '')
                 )
-                for name, excs in self.groups.items()
+                for name, excs in self._groups.items()
             )
         )
 
-    def __call__(self, name=None, raises=None, types=()):
+    def __call__(self, name=None, raises=None, types=None, catch_once=None):
         return _ExceptContext(
             self, name,
             self.raises if raises is None else raises,
-            self.types if not types else types)
+            self.types if types is None else types,
+            self.catch_once if catch_once is None else catch_once)
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        if exc_val is not None and isinstance(exc_val, self.types):
-            self.set(exc_val)
-            return not self.raises
+        return self().__exit__(exc_type, exc_val, exc_tb)
 
-    def set(self, exc, name=None):
+    def set(self, exc, name=None, mark=True):
+        if name not in self._groups:
+            self._groups[name] = []
+        self._groups[name].append(exc)
+        self._excs[name] = exc
+        if mark:
+            exc.__remoteobj_caught__ = name
+
+    def get(self, name=None):
+        return self._excs.get(name)
+
+    def group(self, name=None):
+        return self._groups.get(name)
+
+    def raise_any(self, name=...):
+        exc = self.get(name) if name != ... else next(iter(self.all()), None)
+        if exc is not None:
+            raise exc
+
+    def all(self):
+        return [e for es in self._groups.values() for e in es]
+
+    def clear(self):
+        self._groups.clear()
+        self._excs.clear()
+
+
+class Except(LocalExcept):
+    def __init__(self, *types, **kw):
+        self._local, self._remote = mp.Pipe()
+        super().__init__(*types, **kw)
+
+    def __str__(self):
+        self.pull()
+        return super().__str__()
+
+    def get(self, name=None):
+        self.pull()
+        return super().get(name)
+
+    def group(self, name=None):
+        self.pull()
+        return super().group(name)
+
+    def all(self):
+        self.pull()
+        return super().all()
+
+    def set(self, exc, name=None, mark=True):
         self._remote.send((
             RemoteException(exc) if exc is not None else None, name))
-
-    def get(self, name=None, all=False):
-        self.pull()
-        return self.groups.get(name) if all else self._excs.get(name)
+        if mark:
+            exc.__remoteobj_caught__ = name
 
     def pull(self):
         '''Pull any exceptions through the pipe.'''
         while self._local.poll():
             exc, name = self._local.recv()
-            if name not in self.groups:
-                self.groups[name] = []
-            self.groups[name].append(exc)
-            self._excs[name] = exc
-
-    def raise_any(self, name=None):
-        exc = self.get(name)
-        if exc is not None:
-            raise exc
-
-    @property
-    def all(self):
-        return [e for es in self.groups.values() for e in es]
-
+            super().set(exc, name)
 
 
 class _ExceptContext:
-    def __init__(self, catch, name=None, raises=False, types=()):
+    def __init__(self, catch, name=None, raises=False, types=(), catch_once=True):
         self.catch = catch
         self.name = name
-        self.raises = raises
+        self.raises = raises or not catch_once
         self.types = types
+        self.catch_once = catch_once
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         if exc_val is not None and isinstance(exc_val, self.types):
-            self.catch.set(exc_val, self.name)
+            self.catch.set(exc_val, self.name, self.catch_once)
             return not self.raises
